@@ -49,7 +49,7 @@
 #include "ev-attachment.h"
 #include "ev-image.h"
 
-#if (defined (HAVE_POPPLER_PAGE_RENDER) || defined (HAVE_POPPLER_PAGE_RENDER_FOR_PRINTING)) && (defined (HAVE_CAIRO_PDF) || defined (HAVE_CAIRO_PS))
+#if (defined (HAVE_POPPLER_PAGE_RENDER)) && (defined (HAVE_CAIRO_PDF) || defined (HAVE_CAIRO_PS))
 #define HAVE_CAIRO_PRINT
 #endif
 
@@ -148,19 +148,6 @@ EV_BACKEND_REGISTER_WITH_CODE (PdfDocument, pdf_document,
 								 pdf_document_page_transition_iface_init);
 			 });
 
-static void
-set_rc_data (PdfDocument     *pdf_document,
-	     EvRenderContext *rc)
-{
-	if (rc->data == NULL) {
-		rc->data = poppler_document_get_page (pdf_document->document,
-						      rc->page);
-		rc->destroy = g_object_unref;
-	} else {
-		g_assert (rc->page == poppler_page_get_index (POPPLER_PAGE (rc->data)));
-	}
-}
-
 static void
 pdf_document_search_free (PdfDocumentSearch   *search)
 {
@@ -296,35 +283,45 @@ pdf_document_get_n_pages (EvDocument *document)
 	return poppler_document_get_n_pages (PDF_DOCUMENT (document)->document);
 }
 
-static void
-pdf_document_get_page_size (EvDocument   *document,
-			    int           page,
-			    double       *width,
-			    double       *height)
+static EvPage *
+pdf_document_get_page (EvDocument *document,
+		       gint        index)
 {
 	PdfDocument *pdf_document = PDF_DOCUMENT (document);
 	PopplerPage *poppler_page;
+	EvPage      *page;
 
-	poppler_page = poppler_document_get_page (pdf_document->document, page);
-	poppler_page_get_size (poppler_page, width, height);
+	poppler_page = poppler_document_get_page (pdf_document->document, index);
+	page = ev_page_new (index);
+	page->backend_page = (EvBackendPage)g_object_ref (poppler_page);
+	page->backend_destroy_func = (EvBackendPageDestroyFunc)g_object_unref;
 	g_object_unref (poppler_page);
+
+	return page;
+}
+
+static void
+pdf_document_get_page_size (EvDocument *document,
+			    EvPage     *page,
+			    double     *width,
+			    double     *height)
+{
+	g_return_if_fail (POPPLER_IS_PAGE (page->backend_page));
+	
+	poppler_page_get_size (POPPLER_PAGE (page->backend_page), width, height);
 }
 
 static char *
 pdf_document_get_page_label (EvDocument *document,
-			     int         page)
+			     EvPage     *page)
 {
-	PopplerPage *poppler_page;
 	char *label = NULL;
 
-	poppler_page = poppler_document_get_page (PDF_DOCUMENT (document)->document,
-						  page);
+	g_return_val_if_fail (POPPLER_IS_PAGE (page->backend_page), NULL);
 
-	g_object_get (G_OBJECT (poppler_page),
+	g_object_get (G_OBJECT (page->backend_page),
 		      "label", &label,
 		      NULL);
-	g_object_unref (poppler_page);
-
 	return label;
 }
 
@@ -445,27 +442,12 @@ pdf_document_get_attachments (EvDocument *document)
 }
 
 static cairo_surface_t *
-pdf_document_render (EvDocument      *document,
-		     EvRenderContext *rc)
+pdf_page_render (PopplerPage     *page,
+		 gint             width,
+		 gint             height,
+		 EvRenderContext *rc)
 {
-	PdfDocument *pdf_document;
 	cairo_surface_t *surface;
-	double width_points, height_points;
-	gint width, height;
-
-	pdf_document = PDF_DOCUMENT (document);
-
-	set_rc_data (pdf_document, rc);
-
-	poppler_page_get_size (POPPLER_PAGE (rc->data), &width_points, &height_points);
-
-	if (rc->rotation == 90 || rc->rotation == 270) {
-		width = (int) ((height_points * rc->scale) + 0.5);
-		height = (int) ((width_points * rc->scale) + 0.5);
-	} else {
-		width = (int) ((width_points * rc->scale) + 0.5);
-		height = (int) ((height_points * rc->scale) + 0.5);
-	}
 
 #ifdef HAVE_POPPLER_PAGE_RENDER
 	cairo_t *cr;
@@ -492,7 +474,7 @@ pdf_document_render (EvDocument      *document,
 	}
 	cairo_scale (cr, rc->scale, rc->scale);
 	cairo_rotate (cr, rc->rotation * G_PI / 180.0);
-	poppler_page_render (POPPLER_PAGE (rc->data), cr);
+	poppler_page_render (page, cr);
 	cairo_destroy (cr);
 #else /* HAVE_POPPLER_PAGE_RENDER */
 	GdkPixbuf *pixbuf;
@@ -501,7 +483,7 @@ pdf_document_render (EvDocument      *document,
 				 FALSE, 8,
 				 width, height);
 
-	poppler_page_render_to_pixbuf (POPPLER_PAGE (rc->data),
+	poppler_page_render_to_pixbuf (page,
 				       0, 0,
 				       width, height,
 				       rc->scale,
@@ -511,7 +493,33 @@ pdf_document_render (EvDocument      *document,
 	g_object_unref (pixbuf);
 #endif /* HAVE_POPPLER_PAGE_RENDER */
 
-	return surface;
+	return surface;	
+}
+
+static cairo_surface_t *
+pdf_document_render (EvDocument      *document,
+		     EvRenderContext *rc)
+{
+	PdfDocument *pdf_document;
+	PopplerPage *poppler_page;
+	double width_points, height_points;
+	gint width, height;
+
+	poppler_page = POPPLER_PAGE (rc->page->backend_page);
+
+	poppler_page_get_size (poppler_page,
+			       &width_points, &height_points);
+	
+	if (rc->rotation == 90 || rc->rotation == 270) {
+		width = (int) ((height_points * rc->scale) + 0.5);
+		height = (int) ((width_points * rc->scale) + 0.5);
+	} else {
+		width = (int) ((width_points * rc->scale) + 0.5);
+		height = (int) ((height_points * rc->scale) + 0.5);
+	}
+	
+	return pdf_page_render (poppler_page,
+				width, height, rc);
 }
 
 /* EvDocumentSecurity */
@@ -543,6 +551,7 @@ pdf_document_get_info (EvDocument *document)
 	PopplerPageMode mode;
 	PopplerViewerPreferences view_prefs;
 	PopplerPermissions permissions;
+	EvPage *page;
 
 	info = g_new0 (EvDocumentInfo, 1);
 
@@ -581,13 +590,20 @@ pdf_document_get_info (EvDocument *document)
 		      "linearized", &(info->linearized),
 		      NULL);
 
-	pdf_document_get_page_size(document, 0,
-				   &(info->paper_width),
-				   &(info->paper_height));
+	info->n_pages = ev_document_get_n_pages (document);
 
-	// Convert to mm.
-	info->paper_width = info->paper_width / 72.0f * 25.4f;
-	info->paper_height = info->paper_height / 72.0f * 25.4f;
+	if (info->n_pages > 0) {
+		page = ev_document_get_page (document, 0);
+		ev_document_get_page_size (document, page,
+					   &(info->paper_width),
+					   &(info->paper_height));
+		g_object_unref (page);
+		
+
+		// Convert to mm.
+		info->paper_width = info->paper_width / 72.0f * 25.4f;
+		info->paper_height = info->paper_height / 72.0f * 25.4f;
+	}
 
 	switch (layout) {
 		case POPPLER_PAGE_LAYOUT_SINGLE_PAGE:
@@ -667,8 +683,6 @@ pdf_document_get_info (EvDocument *document)
 		info->permissions |= EV_DOCUMENT_PERMISSIONS_OK_TO_ADD_NOTES;
 	}
 
-	info->n_pages = ev_document_get_n_pages (document);
-
 	if (ev_document_security_has_document_security (EV_DOCUMENT_SECURITY (document))) {
 		/* translators: this is the document security state */
 		info->security = g_strdup (_("Yes"));
@@ -686,6 +700,7 @@ pdf_document_document_iface_init (EvDocumentIface *iface)
 	iface->save = pdf_document_save;
 	iface->load = pdf_document_load;
 	iface->get_n_pages = pdf_document_get_n_pages;
+	iface->get_page = pdf_document_get_page;
 	iface->get_page_size = pdf_document_get_page_size;
 	iface->get_page_label = pdf_document_get_page_label;
 	iface->has_attachments = pdf_document_has_attachments;
@@ -919,7 +934,7 @@ ev_link_dest_from_dest (PdfDocument *pdf_document,
 	}
 
 	if (unimplemented_dest) {
-		g_warning ("Unimplemented named action: %s, please post a "
+		g_warning ("Unimplemented destination: %s, please post a "
 		           "bug report in Evince bugzilla "
 		           "(http://bugzilla.gnome.org) with a testcase.",
 			   unimplemented_dest);
@@ -940,6 +955,8 @@ ev_link_from_action (PdfDocument   *pdf_document,
 	const char   *unimplemented_action = NULL;
 
 	switch (action->type) {
+	        case POPPLER_ACTION_NONE:
+			break;
 	        case POPPLER_ACTION_GOTO_DEST: {
 			EvLinkDest *dest;
 			
@@ -974,8 +991,9 @@ ev_link_from_action (PdfDocument   *pdf_document,
 	}
 	
 	if (unimplemented_action) {
-		g_warning ("Unimplemented action: %s, please post a bug report with a testcase.",
-			   unimplemented_action);
+		g_warning ("Unimplemented action: %s, please post a bug report "
+			   "in Evince bugzilla (http://bugzilla.gnome.org) "
+			   "with a testcase.", unimplemented_action);
 	}
 	
 	link = ev_link_new (action->any.title, ev_action);
@@ -1176,11 +1194,8 @@ pdf_document_images_get_image_mapping (EvDocumentImages *document_images,
 		image_mapping = (PopplerImageMapping *)list->data;
 
 		ev_image_mapping = g_new (EvImageMapping, 1);
-#ifdef HAVE_POPPLER_PAGE_GET_IMAGE
+		
 		ev_image_mapping->image = ev_image_new (page, image_mapping->image_id);
-#else
-		ev_image_mapping->image = ev_image_new_from_pixbuf (image_mapping->image);
-#endif
 		ev_image_mapping->x1 = image_mapping->area.x1;
 		ev_image_mapping->x2 = image_mapping->area.x2;
 		ev_image_mapping->y1 = image_mapping->area.y1;
@@ -1240,7 +1255,7 @@ make_thumbnail_for_page (PdfDocument     *pdf_document,
 
 	pdf_document_thumbnails_get_dimensions (EV_DOCUMENT_THUMBNAILS (pdf_document),
 						rc, &width, &height);
-
+#ifdef POPPLER_WITH_GDK
 	pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, FALSE, 8,
 				 width, height);
 	gdk_pixbuf_fill (pixbuf, 0xffffffff);
@@ -1250,6 +1265,16 @@ make_thumbnail_for_page (PdfDocument     *pdf_document,
 				       width, height,
 				       rc->scale, rc->rotation, pixbuf);
 	ev_document_fc_mutex_unlock ();
+#else
+	cairo_surface_t *surface;
+
+	ev_document_fc_mutex_lock ();
+	surface = pdf_page_render (poppler_page, width, height, rc);
+	ev_document_fc_mutex_unlock ();
+	
+	pixbuf = ev_document_misc_pixbuf_from_surface (surface);
+	cairo_surface_destroy (surface);
+#endif /* POPPLER_WITH_GDK */
 
 	return pixbuf;
 }
@@ -1259,17 +1284,25 @@ pdf_document_thumbnails_get_thumbnail (EvDocumentThumbnails *document_thumbnails
 				       EvRenderContext      *rc, 
 				       gboolean              border)
 {
-	PdfDocument *pdf_document;
+	PdfDocument *pdf_document = PDF_DOCUMENT (document_thumbnails);
 	PopplerPage *poppler_page;
-	GdkPixbuf *pixbuf;
+	GdkPixbuf *pixbuf = NULL;
 	GdkPixbuf *border_pixbuf;
 
-	pdf_document = PDF_DOCUMENT (document_thumbnails);
+	poppler_page = POPPLER_PAGE (rc->page->backend_page);
 
-	poppler_page = poppler_document_get_page (pdf_document->document, rc->page);
-	g_return_val_if_fail (poppler_page != NULL, NULL);
+#ifdef POPPLER_WITH_GDK
+	pixbuf = poppler_page_get_thumbnail_pixbuf (poppler_page);
+#else
+	cairo_surface_t *surface;
+	
+	surface = poppler_page_get_thumbnail (poppler_page);
+	if (surface) {
+		pixbuf = ev_document_misc_pixbuf_from_surface (surface);
+		cairo_surface_destroy (surface);
+	}
+#endif /* POPPLER_WITH_GDK */
 
-	pixbuf = poppler_page_get_thumbnail (poppler_page);
 	if (pixbuf) {
 		/* Rotate provided thumbnail if needed */
 		GdkPixbuf *rotated_pixbuf;
@@ -1289,8 +1322,6 @@ pdf_document_thumbnails_get_thumbnail (EvDocumentThumbnails *document_thumbnails
 		pixbuf = border_pixbuf;
 	}		
 
-	g_object_unref (poppler_page);
-	
 	return pixbuf;
 }
 
@@ -1300,14 +1331,10 @@ pdf_document_thumbnails_get_dimensions (EvDocumentThumbnails *document_thumbnail
 					gint                 *width,
 					gint                 *height)
 {
-	PdfDocument *pdf_document;
 	PopplerPage *poppler_page;
 	gint has_thumb;
 	
-	pdf_document = PDF_DOCUMENT (document_thumbnails);
-	poppler_page = poppler_document_get_page (pdf_document->document, rc->page);
-
-	g_return_if_fail (poppler_page != NULL);
+	poppler_page = POPPLER_PAGE (rc->page->backend_page);
 
 	has_thumb = poppler_page_get_thumbnail_size (poppler_page, width, height);
 
@@ -1327,8 +1354,6 @@ pdf_document_thumbnails_get_dimensions (EvDocumentThumbnails *document_thumbnail
 		*width = *height;
 		*height = temp;
 	}
-	
-	g_object_unref (poppler_page);
 }
 
 static void
@@ -1667,7 +1692,7 @@ pdf_document_file_exporter_do_page (EvFileExporter  *exporter,
 #ifdef HAVE_CAIRO_PRINT
 	gdouble  page_width, page_height;
 	gint     x, y;
-	gboolean rotate, landscape;
+	gboolean rotate;
 	gdouble  width, height;
 	gdouble  pwidth, pheight;
 	gdouble  xscale, yscale;
@@ -1675,7 +1700,7 @@ pdf_document_file_exporter_do_page (EvFileExporter  *exporter,
 
 	g_return_if_fail (pdf_document->print_ctx != NULL);
 
-	poppler_page = poppler_document_get_page (pdf_document->document, rc->page);
+	poppler_page = POPPLER_PAGE (rc->page->backend_page);
 	
 #ifdef HAVE_CAIRO_PRINT
 	x = (ctx->pages_printed % ctx->pages_per_sheet) % ctx->pages_x;
@@ -1688,8 +1713,6 @@ pdf_document_file_exporter_do_page (EvFileExporter  *exporter,
 		rotate = FALSE;
 	}
 
-	landscape = (ctx->paper_width > ctx->paper_height);
-
 	/* Use always portrait mode and rotate when necessary */
 	if (ctx->paper_width > ctx->paper_height) {
 		width = ctx->paper_height;
@@ -1741,7 +1764,7 @@ pdf_document_file_exporter_do_page (EvFileExporter  *exporter,
 	if (rotate) {
 		cairo_matrix_t matrix;
 		
-		cairo_translate (ctx->cr, width, 0);
+		cairo_translate (ctx->cr, (2 * y + 1) * pwidth, 0);
 		cairo_matrix_init (&matrix,
 				   0,  1,
 				   -1,  0,
@@ -1754,13 +1777,7 @@ pdf_document_file_exporter_do_page (EvFileExporter  *exporter,
 			 y * (rotate ? pwidth : pheight));
 	cairo_scale (ctx->cr, xscale, yscale);
 
-#ifdef HAVE_POPPLER_PAGE_RENDER_FOR_PRINTING
 	poppler_page_render_for_printing (poppler_page, ctx->cr);
-#else
-#ifdef HAVE_POPPLER_PAGE_RENDER
-	poppler_page_render (poppler_page, ctx->cr);
-#endif
-#endif
 
 	ctx->pages_printed++;
 			
@@ -1769,8 +1786,6 @@ pdf_document_file_exporter_do_page (EvFileExporter  *exporter,
 	if (ctx->format == EV_FILE_FORMAT_PS)
 		poppler_page_render_to_ps (poppler_page, ctx->ps_file);
 #endif /* HAVE_CAIRO_PRINT */
-	
-	g_object_unref (poppler_page);
 }
 
 static void
@@ -1841,20 +1856,28 @@ pdf_selection_render_selection (EvSelection      *selection,
 				GdkColor         *text,
 				GdkColor         *base)
 {
-	PdfDocument *pdf_document;
+	PopplerPage *poppler_page;
 	double width_points, height_points;
 	gint width, height;
 
-	pdf_document = PDF_DOCUMENT (selection);
-	set_rc_data (pdf_document, rc);
+	poppler_page = POPPLER_PAGE (rc->page->backend_page);
 
-	poppler_page_get_size (POPPLER_PAGE (rc->data),
+	poppler_page_get_size (poppler_page,
 			       &width_points, &height_points);
 	width = (int) ((width_points * rc->scale) + 0.5);
 	height = (int) ((height_points * rc->scale) + 0.5);
 
 #ifdef HAVE_POPPLER_PAGE_RENDER
 	cairo_t *cr;
+	PopplerColor text_color, base_color;
+	
+	text_color.red = text->red;
+	text_color.green = text->green;
+	text_color.blue = text->blue;
+
+	base_color.red = base->red;
+	base_color.green = base->green;
+	base_color.blue = base->blue;
 
 	if (*surface == NULL) {
 		*surface = cairo_image_surface_create (CAIRO_FORMAT_ARGB32,
@@ -1868,13 +1891,13 @@ pdf_selection_render_selection (EvSelection      *selection,
 	memset (cairo_image_surface_get_data (*surface), 0x00,
 		cairo_image_surface_get_height (*surface) *
 		cairo_image_surface_get_stride (*surface));
-	poppler_page_render_selection (POPPLER_PAGE (rc->data),
+	poppler_page_render_selection (poppler_page,
 				       cr,
 				       (PopplerRectangle *)points,
 				       (PopplerRectangle *)old_points,
 				       (PopplerSelectionStyle)style,
-				       text,
-				       base);
+				       &text_color,
+				       &base_color);
 	cairo_destroy (cr);
 #else /* HAVE_POPPLER_PAGE_RENDER */
 	GdkPixbuf *pixbuf;
@@ -1883,7 +1906,7 @@ pdf_selection_render_selection (EvSelection      *selection,
 				 TRUE, 8,
 				 width, height);
 
-	poppler_page_render_selection_to_pixbuf (POPPLER_PAGE (rc->data),
+	poppler_page_render_selection_to_pixbuf (poppler_page,
 						 rc->scale, rc->rotation, pixbuf,
 						 (PopplerRectangle *)points,
 						 (PopplerRectangle *)old_points,
@@ -1903,14 +1926,12 @@ pdf_selection_get_selected_text (EvSelection     *selection,
 				 EvSelectionStyle style,
 				 EvRectangle     *points)
 {
-	PdfDocument *pdf_document = PDF_DOCUMENT (selection);
 	PopplerPage *poppler_page;
 	PopplerRectangle r;
 	double height;
 	char *retval;
 	
-	poppler_page = poppler_document_get_page (pdf_document->document, rc->page);
-	g_return_val_if_fail (poppler_page != NULL, NULL);
+	poppler_page = POPPLER_PAGE (rc->page->backend_page);
 
 	poppler_page_get_size (poppler_page, NULL, &height);
 	r.x1 = points->x1;
@@ -1922,7 +1943,31 @@ pdf_selection_get_selected_text (EvSelection     *selection,
 					(PopplerSelectionStyle)style,
 					&r);
 
-	g_object_unref (poppler_page);
+	return retval;
+}
+
+static GdkRegion *
+create_gdk_region_from_poppler_region (GList *region)
+{
+	GList *l;
+	GdkRegion *retval;
+	
+	retval = gdk_region_new ();
+	
+	for (l = region; l; l = g_list_next (l)) {
+		PopplerRectangle *rectangle;
+		GdkRectangle      rect;
+		
+		rectangle = (PopplerRectangle *)l->data;
+		
+		rect.x = (gint) rectangle->x1;
+		rect.y = (gint) rectangle->y1;
+		rect.width  = (gint) (rectangle->x2 - rectangle->x1);
+		rect.height = (gint) (rectangle->y2 - rectangle->y1);
+		gdk_region_union_with_rect (retval, &rect);
+		
+		poppler_rectangle_free (rectangle);
+	}
 
 	return retval;
 }
@@ -1933,17 +1978,19 @@ pdf_selection_get_selection_region (EvSelection     *selection,
 				    EvSelectionStyle style,
 				    EvRectangle     *points)
 {
-	PdfDocument *pdf_document;
-	GdkRegion *retval;
+	PopplerPage *poppler_page;
+	GdkRegion   *retval;
+	GList       *region;
 
-	pdf_document = PDF_DOCUMENT (selection);
-
-	set_rc_data (pdf_document, rc);
-
-	retval = poppler_page_get_selection_region ((PopplerPage *)rc->data,
+	poppler_page = POPPLER_PAGE (rc->page->backend_page);
+	
+	region = poppler_page_get_selection_region (poppler_page,
 						    rc->scale,
 						    (PopplerSelectionStyle)style,
 						    (PopplerRectangle *) points);
+	retval = create_gdk_region_from_poppler_region (region);
+	g_list_free (region);
+	
 	return retval;
 }
 
@@ -1951,22 +1998,22 @@ static GdkRegion *
 pdf_selection_get_selection_map (EvSelection     *selection,
 				 EvRenderContext *rc)
 {
-	PdfDocument *pdf_document;
 	PopplerPage *poppler_page;
 	PopplerRectangle points;
+	GList *region;
 	GdkRegion *retval;
 
-	pdf_document = PDF_DOCUMENT (selection);
-	poppler_page = poppler_document_get_page (pdf_document->document,
-						  rc->page);
+	poppler_page = POPPLER_PAGE (rc->page->backend_page);
 
 	points.x1 = 0.0;
 	points.y1 = 0.0;
 	poppler_page_get_size (poppler_page, &(points.x2), &(points.y2));
-	retval = poppler_page_get_selection_region (poppler_page, 1.0,
+	
+	region = poppler_page_get_selection_region (poppler_page, 1.0,
 						    POPPLER_SELECTION_GLYPH,
 						    &points);
-	g_object_unref (poppler_page);
+	retval = create_gdk_region_from_poppler_region (region);
+	g_list_free (region);
 
 	return retval;
 }
@@ -2170,17 +2217,17 @@ ev_form_field_from_poppler_field (PopplerFormField *poppler_field)
 
 static GList *
 pdf_document_forms_get_form_fields (EvDocumentForms *document, 
-				    gint             page)
+				    EvPage          *page)
 {
- 	PdfDocument *pdf_document;
  	PopplerPage *poppler_page;
  	GList *retval = NULL;
  	GList *fields;
  	GList *list;
  	double height;
 
- 	pdf_document = PDF_DOCUMENT (document);
- 	poppler_page = poppler_document_get_page (pdf_document->document, page);
+	g_return_val_if_fail (POPPLER_IS_PAGE (page->backend_page), NULL);
+	
+ 	poppler_page = POPPLER_PAGE (page->backend_page);
  	fields = poppler_page_get_form_field_mapping (poppler_page);
  	poppler_page_get_size (poppler_page, NULL, &height);
 
@@ -2201,13 +2248,17 @@ pdf_document_forms_get_form_fields (EvDocumentForms *document,
 		field_mapping->y1 = height - mapping->area.y2;
 		field_mapping->y2 = height - mapping->area.y1;
 		field_mapping->field = ev_field;
-		field_mapping->field->page = page;
+		field_mapping->field->page = EV_PAGE (g_object_ref (page));
+
+		g_object_set_data_full (G_OBJECT (ev_field),
+					"poppler-field",
+					g_object_ref (mapping->field),
+					(GDestroyNotify) g_object_unref);
 		
 		retval = g_list_prepend (retval, field_mapping);
 	}
 	
 	poppler_page_free_form_field_mapping (fields);
-	g_object_unref (poppler_page);
 
 	return g_list_reverse (retval);
 }
@@ -2217,16 +2268,14 @@ pdf_document_forms_form_field_text_get_text (EvDocumentForms *document,
 					     EvFormField     *field)
 	
 {
-	PdfDocument *pdf_document = PDF_DOCUMENT (document);
 	PopplerFormField *poppler_field;
 	gchar *text;
 
-	poppler_field = poppler_document_get_form_field (pdf_document->document, field->id);
+	poppler_field = POPPLER_FORM_FIELD (g_object_get_data (G_OBJECT (field), "poppler-field"));
 	if (!poppler_field)
 		return NULL;
 	
 	text = poppler_form_field_text_get_text (poppler_field);
-	g_object_unref (poppler_field);
 
 	return text;
 }
@@ -2236,14 +2285,12 @@ pdf_document_forms_form_field_text_set_text (EvDocumentForms *document,
 					     EvFormField     *field,
 					     const gchar     *text)
 {
-	PdfDocument *pdf_document = PDF_DOCUMENT (document);
 	PopplerFormField *poppler_field;
 
-	poppler_field = poppler_document_get_form_field (pdf_document->document, field->id);
+	poppler_field = POPPLER_FORM_FIELD (g_object_get_data (G_OBJECT (field), "poppler-field"));
 	if (!poppler_field)
 		return;
 	poppler_form_field_text_set_text (poppler_field, text);
-	g_object_unref (poppler_field);
 }
 
 static void
@@ -2251,31 +2298,27 @@ pdf_document_forms_form_field_button_set_state (EvDocumentForms *document,
 						EvFormField     *field,
 						gboolean         state)
 {
-	PdfDocument *pdf_document = PDF_DOCUMENT (document);
 	PopplerFormField *poppler_field;
 
-	poppler_field = poppler_document_get_form_field (pdf_document->document, field->id);
+	poppler_field = POPPLER_FORM_FIELD (g_object_get_data (G_OBJECT (field), "poppler-field"));
 	if (!poppler_field)
 		return;
 	
 	poppler_form_field_button_set_state (poppler_field, state);
-	g_object_unref (poppler_field);
 }
 
 static gboolean
 pdf_document_forms_form_field_button_get_state (EvDocumentForms *document, 
 						EvFormField     *field)
 {
-	PdfDocument *pdf_document = PDF_DOCUMENT (document);
 	PopplerFormField *poppler_field;
 	gboolean state;
 
-	poppler_field = poppler_document_get_form_field (pdf_document->document, field->id);
+	poppler_field = POPPLER_FORM_FIELD (g_object_get_data (G_OBJECT (field), "poppler-field"));
 	if (!poppler_field)
 		return FALSE;
 
 	state = poppler_form_field_button_get_state (poppler_field);
-	g_object_unref (poppler_field);
 
 	return state;
 }
@@ -2285,16 +2328,14 @@ pdf_document_forms_form_field_choice_get_item (EvDocumentForms *document,
 					       EvFormField     *field,
 					       gint             index)
 {
-	PdfDocument *pdf_document = PDF_DOCUMENT (document);
 	PopplerFormField *poppler_field;
 	gchar *text;
 
-	poppler_field = poppler_document_get_form_field (pdf_document->document, field->id);
+	poppler_field = POPPLER_FORM_FIELD (g_object_get_data (G_OBJECT (field), "poppler-field"));
 	if (!poppler_field)
 		return NULL;
 
 	text = poppler_form_field_choice_get_item (poppler_field, index);
-	g_object_unref (poppler_field);
 
 	return text;
 }
@@ -2303,16 +2344,14 @@ static int
 pdf_document_forms_form_field_choice_get_n_items (EvDocumentForms *document, 
 						  EvFormField     *field)
 {
-	PdfDocument *pdf_document = PDF_DOCUMENT (document);
 	PopplerFormField *poppler_field;
 	gint n_items;
 
-	poppler_field = poppler_document_get_form_field (pdf_document->document, field->id);
+	poppler_field = POPPLER_FORM_FIELD (g_object_get_data (G_OBJECT (field), "poppler-field"));
 	if (!poppler_field)
 		return -1;
 	
 	n_items = poppler_form_field_choice_get_n_items (poppler_field);
-	g_object_unref (poppler_field);
 
 	return n_items;
 }
@@ -2322,16 +2361,14 @@ pdf_document_forms_form_field_choice_is_item_selected (EvDocumentForms *document
 						       EvFormField     *field,
 						       gint             index)
 {
-	PdfDocument *pdf_document = PDF_DOCUMENT (document);
 	PopplerFormField *poppler_field;
 	gboolean selected;
 
-	poppler_field = poppler_document_get_form_field (pdf_document->document, field->id);
+	poppler_field = POPPLER_FORM_FIELD (g_object_get_data (G_OBJECT (field), "poppler-field"));
 	if (!poppler_field)
 		return FALSE;
 
 	selected = poppler_form_field_choice_is_item_selected (poppler_field, index);
-	g_object_unref (poppler_field);
 
 	return selected;
 }
@@ -2341,15 +2378,13 @@ pdf_document_forms_form_field_choice_select_item (EvDocumentForms *document,
 						  EvFormField     *field,
 						  gint             index)
 {
-	PdfDocument *pdf_document = PDF_DOCUMENT (document);
 	PopplerFormField *poppler_field;
 
-	poppler_field = poppler_document_get_form_field (pdf_document->document, field->id);
+	poppler_field = POPPLER_FORM_FIELD (g_object_get_data (G_OBJECT (field), "poppler-field"));
 	if (!poppler_field)
 		return;
 
 	poppler_form_field_choice_select_item (poppler_field, index);
-	g_object_unref (poppler_field);
 }
 
 static void
@@ -2357,30 +2392,26 @@ pdf_document_forms_form_field_choice_toggle_item (EvDocumentForms *document,
 						  EvFormField     *field,
 						  gint             index)
 {
-	PdfDocument *pdf_document = PDF_DOCUMENT (document);
 	PopplerFormField *poppler_field;
 
-	poppler_field = poppler_document_get_form_field (pdf_document->document, field->id);
+	poppler_field = POPPLER_FORM_FIELD (g_object_get_data (G_OBJECT (field), "poppler-field"));
 	if (!poppler_field)
 		return;
 
 	poppler_form_field_choice_toggle_item (poppler_field, index);
-	g_object_unref (poppler_field);
 }
 
 static void
 pdf_document_forms_form_field_choice_unselect_all (EvDocumentForms *document, 
 						   EvFormField     *field)
 {
-	PdfDocument *pdf_document = PDF_DOCUMENT (document);
 	PopplerFormField *poppler_field;
 
-	poppler_field = poppler_document_get_form_field (pdf_document->document, field->id);
+	poppler_field = POPPLER_FORM_FIELD (g_object_get_data (G_OBJECT (field), "poppler-field"));
 	if (!poppler_field)
 		return;
 	
 	poppler_form_field_choice_unselect_all (poppler_field);
-	g_object_unref (poppler_field);
 }
 
 static void
@@ -2388,31 +2419,27 @@ pdf_document_forms_form_field_choice_set_text (EvDocumentForms *document,
 					       EvFormField     *field,
 					       const gchar     *text)
 {
-	PdfDocument *pdf_document = PDF_DOCUMENT (document);
 	PopplerFormField *poppler_field;
 
-	poppler_field = poppler_document_get_form_field (pdf_document->document, field->id);
+	poppler_field = POPPLER_FORM_FIELD (g_object_get_data (G_OBJECT (field), "poppler-field"));
 	if (!poppler_field)
 		return;
 	
 	poppler_form_field_choice_set_text (poppler_field, text);
-	g_object_unref (poppler_field);
 }
 
 static gchar *
 pdf_document_forms_form_field_choice_get_text (EvDocumentForms *document,
 					       EvFormField     *field)
 {
-	PdfDocument *pdf_document = PDF_DOCUMENT (document);
 	PopplerFormField *poppler_field;
 	gchar *text;
 
-	poppler_field = poppler_document_get_form_field (pdf_document->document, field->id);
+	poppler_field = POPPLER_FORM_FIELD (g_object_get_data (G_OBJECT (field), "poppler-field"));
 	if (!poppler_field)
 		return NULL;
 
 	text = poppler_form_field_choice_get_text (poppler_field);
-	g_object_unref (poppler_field);
 
 	return text;
 }
