@@ -25,6 +25,9 @@
 #include "ev-utils.h"
 #include "ev-file-helpers.h"
 #include "ev-document-factory.h"
+#ifdef ENABLE_DBUS
+#include "ev-media-player-keys.h"
+#endif /* ENABLE_DBUS */
 #include "totem-scrsaver.h"
 
 #include <glib.h>
@@ -59,12 +62,14 @@ struct _EvApplication {
 
 	gchar *last_chooser_uri;
 
-#ifdef WITH_GTK_PRINT
+#ifdef ENABLE_DBUS
+	EvMediaPlayerKeys *keys;
+#endif /* ENABLE_DBUS */
+
 	GtkPrintSettings *print_settings;
 #if GTK_CHECK_VERSION (2, 11, 0)
 	gchar            *print_settings_file;
 #endif
-#endif
 };
 
 struct _EvApplicationClass {
@@ -638,7 +643,6 @@ ev_application_shutdown (EvApplication *application)
 		application->toolbars_file = NULL;
 	}
 
-#ifdef WITH_GTK_PRINT
 #if GTK_CHECK_VERSION (2, 11, 0)
 	if (application->print_settings_file) {
 		if (application->print_settings) {
@@ -648,7 +652,7 @@ ev_application_shutdown (EvApplication *application)
 						    application->print_settings_file,
 						    &error);
 			if (error) {
-				g_warning (error->message);
+				g_warning ("%s", error->message);
 				g_error_free (error);
 			}
 
@@ -665,7 +669,6 @@ ev_application_shutdown (EvApplication *application)
 		application->print_settings = NULL;
 	}
 #endif /* GTK 2.11.0 */
-#endif /* WITH_GTK_PRINT */
 	
 	g_free (application->last_chooser_uri);
 	g_object_unref (application);
@@ -719,6 +722,10 @@ ev_application_init (EvApplication *ev_application)
 
 	egg_toolbars_model_set_flags (ev_application->toolbars_model, 0,
 				      EGG_TB_MODEL_NOT_REMOVABLE);
+
+#ifdef ENABLE_DBUS
+	ev_application->keys = ev_media_player_keys_new ();
+#endif /* ENABLE_DBUS */
 }
 
 /**
@@ -748,6 +755,26 @@ ev_application_get_windows (EvApplication *application)
 	return windows;
 }
 
+/**
+ * ev_application_get_media_keys:
+ * @application: The instance of the application.
+ *
+ * It gives you access to the media player keys handler object.
+ *
+ * Returns: A #EvMediaPlayerKeys.
+ */
+GObject
+*ev_application_get_media_keys (EvApplication *application)
+{
+#ifdef ENABLE_DBUS
+	if (!application->keys)
+		return NULL;
+	return g_object_ref (G_OBJECT (application->keys));
+#else
+	return NULL;
+#endif /* ENABLE_DBUS */
+}
+
 EggToolbarsModel *
 ev_application_get_toolbars_model (EvApplication *application)
 {
@@ -788,7 +815,6 @@ ev_application_screensaver_disable (EvApplication *application)
 		totem_scrsaver_disable (application->scr_saver);	
 }
 
-#ifdef WITH_GTK_PRINT
 GtkPrintSettings *
 ev_application_get_print_settings (EvApplication *application)
 {
@@ -808,7 +834,7 @@ ev_application_get_print_settings (EvApplication *application)
 			gtk_print_settings_new_from_file (application->print_settings_file, &error);
 		
 		if (error) {
-			g_warning (error->message);
+			g_warning ("%s", error->message);
 			g_error_free (error);
 		} else {
 			return application->print_settings;
@@ -835,4 +861,4 @@ ev_application_set_print_settings (EvApplication    *application,
 	
 	application->print_settings = g_object_ref (settings);
 }
-#endif /* WITH_GTK_PRINT */
+
