@@ -29,17 +29,11 @@
 
 #include <glib/gerror.h>
 #include <glib-object.h>
-
-#ifdef WITH_GTK_PRINT
 #include <gtk/gtkprintsettings.h>
-#endif
 
 #include "ev-window.h"
 
 #include "egg-toolbars-model.h"
-#ifndef HAVE_GTK_RECENT
-#include "egg-recent-model.h"
-#endif
 
 G_BEGIN_DECLS
 
@@ -75,6 +69,7 @@ void              ev_application_open_uri_at_dest    (EvApplication   *applicati
 						      GdkScreen       *screen,
 						      EvLinkDest      *dest,
 						      EvWindowRunMode  mode,
+						      const gchar     *search_string,
 						      gboolean         unlink_temp_file,
 						      const gchar     *print_settings, 
 						      guint32          timestamp);
@@ -83,22 +78,18 @@ void	          ev_application_open_uri_list       (EvApplication   *application,
 						      GdkScreen       *screen,
     						      guint32          timestamp);
 GList		 *ev_application_get_windows	     (EvApplication   *application);
+GObject		 *ev_application_get_media_keys	     (EvApplication   *application);
 
 EggToolbarsModel *ev_application_get_toolbars_model  (EvApplication   *application);
 void              ev_application_save_toolbars_model (EvApplication   *application);
-#ifndef HAVE_GTK_RECENT
-EggRecentModel   *ev_application_get_recent_model    (EvApplication   *application);
-#endif
 void 		  ev_application_set_chooser_uri     (EvApplication   *application, 
 						      const gchar     *uri);
 const gchar	 *ev_application_get_chooser_uri     (EvApplication   *application);
 void		  ev_application_screensaver_enable  (EvApplication   *application);
 void		  ev_application_screensaver_disable (EvApplication   *application);
-#ifdef WITH_GTK_PRINT
 GtkPrintSettings *ev_application_get_print_settings  (EvApplication   *application);
 void              ev_application_set_print_settings  (EvApplication   *application,
 						      GtkPrintSettings *settings);
-#endif
 
 G_END_DECLS
 
