@@ -260,13 +260,10 @@ add_range (EvSidebarThumbnails *sidebar_thumbnails,
 				    -1);
 
 		if (job == NULL && !thumbnail_set) {
-			EvRenderContext *rc;
-
-			rc = ev_render_context_new (priv->rotation, page,
+			job = ev_job_thumbnail_new (priv->document,
+						    page, priv->rotation,
 						    get_scale_for_page (sidebar_thumbnails, page));
-			job = ev_job_thumbnail_new (priv->document, rc);
 			ev_job_queue_add_job (EV_JOB (job), EV_JOB_PRIORITY_HIGH);
-			g_object_unref (rc);
 			
 			g_object_set_data_full (G_OBJECT (job), "tree_iter",
 						gtk_tree_iter_copy (&iter),
@@ -406,7 +403,8 @@ ev_sidebar_thumbnails_set_loading_icon (EvSidebarThumbnails *sidebar_thumbnails)
 	if (sidebar_thumbnails->priv->loading_icon)
 		g_object_unref (sidebar_thumbnails->priv->loading_icon);
 
-	if (sidebar_thumbnails->priv->document) {
+	if (sidebar_thumbnails->priv->document &&
+	    sidebar_thumbnails->priv->n_pages > 0) {
 		gint width, height;
 
 		/* We get the dimensions of the first page so that we can make a blank
@@ -437,7 +435,8 @@ ev_sidebar_thumbnails_refresh (EvSidebarThumbnails *sidebar_thumbnails,
 	sidebar_thumbnails->priv->rotation = rotation;
 	ev_sidebar_thumbnails_set_loading_icon (sidebar_thumbnails);
 
-	if (sidebar_thumbnails->priv->document == NULL)
+	if (sidebar_thumbnails->priv->document == NULL ||
+	    sidebar_thumbnails->priv->n_pages <= 0)
 		return;
 
 	ev_sidebar_thumbnails_clear_model (sidebar_thumbnails);
