@@ -73,9 +73,10 @@ void		ev_view_set_document	  (EvView         *view,
 			   		   EvDocument     *document);
 void 		ev_view_set_loading       (EvView 	  *view,
 				           gboolean        loading);
-
 /* Clipboard */
 void		ev_view_copy		  (EvView         *view);
+void            ev_view_copy_link_address (EvView         *view,
+					   EvLinkAction   *action);
 void		ev_view_select_all	  (EvView         *view);
 gboolean        ev_view_get_has_selection (EvView         *view);
 
@@ -125,6 +126,8 @@ void            ev_view_find_previous        (EvView         *view);
 void            ev_view_search_changed       (EvView         *view);
 void     	ev_view_set_highlight_search (EvView         *view,
 					      gboolean        value);
+void            ev_view_find_cancel          (EvView         *view);
+
 /* Status */
 const char     *ev_view_get_status        (EvView         *view);
 const char     *ev_view_get_find_status   (EvView         *view);
@@ -145,6 +148,9 @@ gchar*         ev_view_page_label_from_dest (EvView *view, EvLinkDest *dest);
 
 void           ev_view_update_view_size (EvView *view, GtkScrolledWindow *scrolled_window);
 
+void	       ev_view_autoscroll_start   (EvView *view);
+void           ev_view_autoscroll_stop    (EvView *view);
+
 G_END_DECLS
 
 #endif /* __EV_VIEW_H__ */
