@@ -82,12 +82,17 @@ void		ev_window_open_uri	  (EvWindow       *ev_window,
 					   gboolean        unlink_temp_file,
 					   const gchar    *print_settings);
 gboolean	ev_window_is_empty	  (const EvWindow *ev_window);
-
-#ifdef WITH_PRINT
 void		ev_window_print_range     (EvWindow       *ev_window,
 					   int             first_page,
 					   int		   last_page);
-#endif
+
+void		ev_window_go_last_page	  (EvWindow       *ev_window);
+void		ev_window_go_first_page	  (EvWindow       *ev_window);
+void		ev_window_go_next_page	  (EvWindow       *ev_window);
+void		ev_window_go_previous_page(EvWindow       *ev_window);
+void		ev_window_start_presentation
+					  (EvWindow       *ev_window);
+
 G_END_DECLS
 
 #endif /* !EV_WINDOW_H */
